@@ -19,9 +19,12 @@ import (
 	"net/http"
 	"os"
 	"sort"
+	"strconv"
 	"strings"
 	"sync/atomic"
 	"time"
+
+	"github.com/OpenPrinting/goipp"
 )
 
 // UsbTransport implements HTTP transport functionality over USB
@@ -76,6 +79,7 @@ func NewUsbTransport(desc UsbDeviceDesc) (*UsbTransport, error) {
 		Debug(' ', "===============================").
 		Info('+', "%s: added %s", transport.addr, transport.info.ProductName).
 		Debug(' ', "Device info:").
+		Debug(' ', "  USB Port:      %d", transport.info.PortNum).
 		Debug(' ', "  Ident:         %s", transport.info.Ident()).
 		Debug(' ', "  Manufacturer:  %s", transport.info.Manufacturer).
 		Debug(' ', "  Product:       %s", transport.info.ProductName).
@@ -84,23 +88,7 @@ func NewUsbTransport(desc UsbDeviceDesc) (*UsbTransport, error) {
 		Debug(' ', "  BasicCaps:     %s", transport.info.BasicCaps).
 		Nl(LogDebug)
 
-	log.Debug(' ', "Device quirks:")
-	for _, quirks := range transport.quirks {
-		log.Debug(' ', "  from [%s] (%s)", quirks.Model, quirks.Origin)
-		log.Debug(' ', "    blacklist = %v", quirks.Blacklist)
-		log.Debug(' ', "    usb-max-interfaces = %v", quirks.UsbMaxInterfaces)
-		log.Debug(' ', "    disable-fax = %v", quirks.DisableFax)
-		log.Debug(' ', "    init-delay = %s", quirks.InitDelay)
-		log.Debug(' ', "    request-delay = %s", quirks.RequestDelay)
-		if quirks.ResetMethod != QuirksResetUnset {
-			log.Debug(' ', "    init-reset = %s", quirks.ResetMethod)
-		}
-		log.Debug(' ', "    ignore-ipp-status= %v", quirks.IgnoreIppStatus)
-
-		for name, value := range quirks.HTTPHeaders {
-			log.Debug(' ', "    http-%s = %q", strings.ToLower(name), value)
-		}
-	}
+	transport.dumpQuirks(log)
 	log.Nl(LogDebug)
 
 	transport.dumpUSBparams(log)
@@ -131,7 +119,7 @@ func NewUsbTransport(desc UsbDeviceDesc) (*UsbTransport, error) {
 	}
 
 	// Hard-reset the device, if needed
-	if transport.quirks.GetResetMethod() == QuirksResetHard {
+	if transport.quirks.GetInitReset() == QuirksResetHard {
 		transport.log.Debug(' ', "Doing USB HARD RESET")
 		dev.Reset()
 	}
@@ -182,6 +170,85 @@ ERROR:
 	return nil, err
 }
 
+// Dump quirks to the UsbTransport's log
+func (transport *UsbTransport) dumpQuirks(log *LogMessage) {
+	type dmp struct {
+		name, val string
+		origin    *Quirks
+	}
+
+	dump := []dmp{}
+
+	var v interface{}
+	var o *Quirks
+
+	// Collect parameters
+	v, o = transport.quirks.GetBlacklistOrigin()
+	dump = append(dump, dmp{"blacklist", fmt.Sprintf("%v", v), o})
+
+	v, o = transport.quirks.GetBuggyIppRspOrigin()
+	dump = append(dump, dmp{"buggy-ipp-responses", fmt.Sprintf("%s", v), o})
+
+	v, o = transport.quirks.GetDisableFaxOrigin()
+	dump = append(dump, dmp{"disable-fax", fmt.Sprintf("%v", v), o})
+
+	v, o = transport.quirks.GetIgnoreIppStatusOrigin()
+	dump = append(dump, dmp{"ignore-ipp-status", fmt.Sprintf("%v", v), o})
+
+	v, o = transport.quirks.GetInitDelayOrigin()
+	dump = append(dump, dmp{"init-delay", fmt.Sprintf("%v", v), o})
+
+	v, o = transport.quirks.GetInitResetOrigin()
+	dump = append(dump, dmp{"init-reset", fmt.Sprintf("%s", v), o})
+
+	v, o = transport.quirks.GetRequestDelayOrigin()
+	dump = append(dump, dmp{"request-delay", fmt.Sprintf("%v", v), o})
+
+	v, o = transport.quirks.GetUsbMaxInterfacesOrigin()
+	dump = append(dump, dmp{"usb-max-interfaces", fmt.Sprintf("%v", v), o})
+
+	// Add HTTP headers
+	for _, q := range transport.quirks {
+		for name, value := range q.HTTPHeaders {
+			d := dmp{
+				name:   "http-" + strings.ToLower(name),
+				val:    fmt.Sprintf("%q", value),
+				origin: q,
+			}
+			dump = append(dump, d)
+		}
+	}
+
+	// Skip no-origin (i.e., actually unset) quirks
+	cnt := 0
+	for _, d := range dump {
+		if d.origin != nil {
+			dump[cnt] = d
+			cnt++
+		}
+	}
+
+	dump = dump[:cnt]
+
+	// Sort by name
+	sort.Slice(dump, func(i, j int) bool {
+		return dump[i].name < dump[j].name
+	})
+
+	// And write to the log
+	log.Debug(' ', "Device quirks:")
+
+	origin := (*Quirks)(nil)
+	for _, d := range dump {
+		if origin != d.origin {
+			origin = d.origin
+			log.Debug(' ', "  from [%s] (%s):",
+				origin.Model, origin.Origin)
+		}
+		log.Debug(' ', "    %s = %s", d.name, d.val)
+	}
+}
+
 // Dump USB stack parameters to the UsbTransport's log
 func (transport *UsbTransport) dumpUSBparams(log *LogMessage) {
 	const usbParamsDir = "/sys/module/usbcore/parameters"
@@ -401,8 +468,13 @@ func (transport *UsbTransport) RoundTripWithSession(session int,
 	switch {
 	case outreq.ContentLength <= 0:
 		// Nothing to do
-		transport.log.HTTPDebug('>', session,
-			"body is chunked, sending as is")
+		if outreq.ContentLength < 0 {
+			transport.log.HTTPDebug('>', session,
+				"body is chunked, sending as is")
+		} else {
+			transport.log.HTTPDebug('>', session,
+				"body is empty, sending as is")
+		}
 
 	case outreq.ContentLength < 16384:
 		// Body is small, prefetch it before sending to USB
@@ -471,6 +543,12 @@ func (transport *UsbTransport) RoundTripWithSession(session int,
 		conn:    conn,
 	}
 
+	// Optionally sanitize IPP response
+	if transport.quirks.GetBuggyIppRsp() == QuirksBuggyIppRspSanitize &&
+		resp.Header.Get("Content-Type") == "application/ipp" {
+		transport.sanitizeIppResponse(session, resp)
+	}
+
 	// Log the response
 	if resp != nil {
 		transport.log.Begin().
@@ -482,6 +560,58 @@ func (transport *UsbTransport) RoundTripWithSession(session int,
 	return resp, nil
 }
 
+// sanitizeIppResponse attempts to sanitize IPP response from device
+func (transport *UsbTransport) sanitizeIppResponse(session int,
+	resp *http.Response) {
+	// Try to prefetch IPP part of message
+	buf := &bytes.Buffer{}
+	buf2 := &bytes.Buffer{}
+
+	tee := io.TeeReader(resp.Body, buf)
+	msg := goipp.Message{}
+	err := msg.DecodeEx(tee, goipp.DecoderOptions{EnableWorkarounds: true})
+	if err != nil {
+		transport.log.HTTPDebug(' ', session,
+			"IPP sanitize: decode: %s", err)
+		goto REPLACE
+	}
+
+	// If backup copy decodes without any options, no need to sanitize
+	if msg2 := (goipp.Message{}); msg2.DecodeBytes(buf.Bytes()) == nil {
+		transport.log.HTTPDebug(' ', session,
+			"IPP sanitize: not needed")
+		goto REPLACE
+	}
+
+	// Re-encode the message correctly
+	err = msg.Encode(buf2)
+	if err != nil {
+		transport.log.HTTPDebug(' ', session,
+			"IPP sanitize: encode: %s", err)
+		goto REPLACE
+	}
+
+	// Replace buffer, adjust resp.ContentLength
+	if resp.ContentLength != -1 {
+		resp.ContentLength += int64(buf2.Len() - buf.Len())
+
+		resp.Header.Set("Content-Length",
+			strconv.FormatInt(resp.ContentLength, 10))
+
+		transport.log.HTTPDebug(' ', session,
+			"IPP sanitize: %d bytes replaced with %d",
+			buf.Len(), buf2.Len())
+	}
+
+	buf = buf2
+
+	// Replace consumed part of message with re-coded or
+	// saved backup copy
+REPLACE:
+	wrap := resp.Body.(*usbResponseBodyWrapper)
+	wrap.preBody = buf
+}
+
 // usbRequestBodyWrapper wraps http.Request.Body, adding
 // data path instrumentation
 type usbRequestBodyWrapper struct {
@@ -522,6 +652,7 @@ func (wrap *usbRequestBodyWrapper) Close() error {
 type usbResponseBodyWrapper struct {
 	log     *Logger       // Device's logger
 	session int           // HTTP session, for logging
+	preBody *bytes.Buffer // Data inserted before body, if not nil
 	body    io.ReadCloser // Response.body
 	conn    *usbConn      // Underlying USB connection
 	count   int           // Total count of received bytes
@@ -530,6 +661,10 @@ type usbResponseBodyWrapper struct {
 
 // Read from usbResponseBodyWrapper
 func (wrap *usbResponseBodyWrapper) Read(buf []byte) (int, error) {
+	if wrap.preBody != nil && wrap.preBody.Len() > 0 {
+		return wrap.preBody.Read(buf)
+	}
+
 	n, err := wrap.body.Read(buf)
 	wrap.count += n
 
@@ -606,7 +741,7 @@ func (transport *UsbTransport) openUsbConn(
 	}
 
 	// Soft-reset interface, if needed
-	if quirks.GetResetMethod() == QuirksResetSoft {
+	if quirks.GetInitReset() == QuirksResetSoft {
 		transport.log.Debug(' ', "USB[%d]: doing SOFT_RESET", index)
 		err = conn.iface.SoftReset()
 		if err != nil {
